/*
The Next Generation Beacons software include file - Management.c

Copyright
  The OZ7IGY Beacon Next Generation Beacon platform implementation
    Management.c/h
    Poul-Erik Hansen, OZ1CKG and Bo Hansen, OZ2M

This file is part of Next Generation Beacons - VCOPLL v. 2

VCOPLL is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

VCOPLL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details:
            www.gnu.org/licenses
*/

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <ctype.h>
#include <avr/pgmspace.h>

#include "VCOPLL.h"                             // Own include files
#include "Management.h"

char mgmtFrame[100];
uint8_t mgmtCharCount;
char mgmtChar;

static char version[] = "2.1.1";

// ------------------- Command parse module ------------------------------------------------------------------------------
/*
   Functionality
       Verifies and processes management strings and puts
       valid data in EEPROM
       If data is changed the DataChanged flag is set
       All input strings are in lower case
       All output/response strings are in upper case

   Input
       string_to_parse - (pointer) to the string to parse

   Output
       (Processed data in EEPROM)

*/
void CommandPaser(const char *string_to_parse)                                           // Management command parser, processes command, return 1 if "done" management otherwise return 0
{
    // Static response strings used by the "rd act" command
    static const char cmdWrEEPROM[] = "wr eep ";                                           // Write command to EEPROM: wr eep address value, where address is three bytes and value is one byte all in hex
    static const char cmdRdEEPROMAll1[] = "rd eep *1";                                     // Read all EEPROM command: rd eep *1
    static const char cmdRdEEPROM[] = "rd eep ";                                           // Read specific address command from EEPROM: rd eep address, where address is three bytes in hex

    static const char eeprom_wr_fl[] = "wr fl ";                                           // Write function latch command to EEPROM: wr fl FL
    static const char eeprom_wr_nc[] = "wr nc ";                                           // Write N counter latch command to EEPROM: wr nc NCOUNTER
    static const char eeprom_wr_rc[] = "wr rc ";                                           // Write R counter latch command to EEPROM: wr rc RCOUNTER
    static const char cmdRdAct[] = "rd act";                                               // Read active settings command: rd act
    static const char cmdRdLock[] = "rd lock";                                             // Read lock command: rd lock
    static const char cmdRdVer[] = "rd ver";                                               // Show version command: rd ver

    static const char cmdWrDefaults[] = "wr defaults";                                     // Write default values to EEPROM: wr defaults
    static const char cmdLoad[] = "load";                                                  // (Re)load changes to PLL from EEPROM: load

    uint32_t funcLatch, nCount, rCount;                                                    // variable holding a parsed function latch, N counter and R counter parsed values
    uint16_t adr, data;                                                                    // Input address and data value
    int16_t i, j;                                                                          // Variable counters

    // wr eep ADDRESS VALUE
    if (0 == strncmp(cmdWrEEPROM, string_to_parse, sizeof(cmdWrEEPROM) - 1))               // Write command check
    {
        if (2 == sscanf(&string_to_parse[sizeof(cmdWrEEPROM) - 1], "%X %X", &adr, &data))    // If both address and value are present and valid then write data to EEPROM
            if ((adr <= 0xFF) && (data <= 0xFF))
            {
                eeprom_write_byte((uint8_t*) adr, data);                                         // Write the data at the specific EEPROM address
                printf("OK\n");
            }
    }
    else

        // rd eep *1
        if (0 == strncmp(cmdRdEEPROMAll1, string_to_parse, sizeof(cmdRdEEPROMAll1)))           // Read all command check 1 byte at a time
        {
            printf("EEPROM 1 BYTE\n");
            printf("=============\n");

            for ( i = 0 ; i <= EEPROM_Adr_HighestUsed; i++)                                                            // Send all EEPROM data to management program, one address at a time, 511 is max EEPROM addressed used
            {
                data = eeprom_read_byte((uint8_t*) i);                                                                 // Get the data at the specific EEPROM address
                printf("%04X: %02X\n", i, data);                                                                       // Send specific address EEPROM data to management program
            }
        }
        else

            // re eep ADDRESS    Must be located after rd eep *1
            if (0 == strncmp(cmdRdEEPROM, string_to_parse, sizeof(cmdRdEEPROM) - 1))                                   // Read command check
            {
                if (1 == sscanf(&string_to_parse[sizeof(cmdRdEEPROM) - 1], "%X", &adr))                                // If address is valid then read data from EEPROM
                    if (adr <= 0xFF)
                    {
                        data = eeprom_read_byte((uint8_t*) adr);                                                       // Get the data at the specific EEPROM address
                        printf("%02X\n", data);                                                                        // Send specific address EEPROM data to management program
                    }
            }
            else

                // wr fl FUNCLATCH
                if (0 == strncmp(eeprom_wr_fl, string_to_parse, sizeof(eeprom_wr_fl) - 1))                             // Write function latch
                {
                    if (1 == sscanf(&string_to_parse[sizeof(eeprom_wr_fl) - 1], "%lX", &funcLatch))                    // Check if funcLatch is present
                    {
                        for (i = 2; i >= 0; i--)
                        {
                            eeprom_write_byte((uint8_t*) (EEPROM_Adr_FuncLatch + i), funcLatch & 0xFF);
                            funcLatch = funcLatch >> 8;
                        }
                        printf("OK\n");
                    }
                }
                else

                    // wr nc NCOUNT
                    if (0 == strncmp(eeprom_wr_nc, string_to_parse, sizeof(eeprom_wr_nc) - 1))                         // Write N counter
                    {
                        if (1 == sscanf(&string_to_parse[sizeof(eeprom_wr_nc) - 1], "%lX", &nCount))                   // Check if nCounter is present
                        {
                            for (i = 2; i >= 0; i--)
                            {
                                eeprom_write_byte((uint8_t*) (EEPROM_Adr_NCounter + i), nCount & 0xFF);
                                nCount = nCount >> 8;
                            }
                            printf("OK\n");
                        }
                    }
                    else

                        // wr rc RCOUNT
                        if (0 == strncmp(eeprom_wr_rc, string_to_parse, sizeof(eeprom_wr_rc) - 1))                     // Write N counter
                        {
                            if (1 == sscanf(&string_to_parse[sizeof(eeprom_wr_rc) - 1], "%lX", &rCount))               // Check if nCounter is present
                            {
                                for (i = 2; i >= 0; i--)
                                {
                                    eeprom_write_byte((uint8_t*) (EEPROM_Adr_RCounter + i), rCount & 0xFF);
                                    rCount = rCount >> 8;
                                }
                                printf("OK\n");
                            }
                        }
                        else

                            // rd act
                            if (0 == strncmp(cmdRdAct, string_to_parse, sizeof(cmdRdAct)))                             // Read active settings parameters command check
                            {
                                printf("ACTIVE PARAMETERS\n");
                                printf("=================\n");

                                printf("LATCHES       2          1\n");
                                printf("-------    32109876 54321098 76543210 (hex)\n");

                                printf("Function : ");
                                for (i = 0; i < 3; i++)
                                {
                                    for (j = 7; j >= 0; j--)
                                        printf("%d", ((functionLatch[i] >> j) & 1));
                                    printf(" ");
                                }
                                for (i = 0; i < 3; i++)
                                    printf("%02X", functionLatch[i]);
                                printf("\n");

                                printf("N counter: ");
                                for (i = 0; i < 3; i++)
                                {
                                    for (j = 7; j >= 0; j--)
                                        printf("%d", ((nCounter[i] >> j) & 1));
                                    printf(" ");
                                }
                                for (i = 0; i < 3; i++)
                                    printf("%02X", nCounter[i]);
                                printf("\n");

                                printf("R counter: ");
                                for (i = 0; i < 3; i++)
                                {
                                    for (j = 7; j >= 0; j--)
                                        printf("%d", ((rCounter[i] >> j) & 1));
                                    printf(" ");
                                }
                                for (i = 0; i < 3; i++)
                                    printf("%02X", rCounter[i]);
                                printf("\n\n");

                                printf("DELAYS AND LOCK\n");
                                printf("---------------\n");
                                printf("Boot delay [ms]          : %d\n", bootDelay);
                                printf("SPI pulse delay [us]     : %d\n", spiPulseDelay);
                                printf("VCO setup delay (*5) [ms]: %d -> %d\n", vcoSetupDelay, 5 * vcoSetupDelay);
                                printf("Main loop delay (*5) [ms]: %d -> %d\n", mainLoopDelay, 5 * mainLoopDelay);
                                printf("Prompt lock status       : %d", promptLockStatus);
                                printf("\n");
                            }
                            else

                                // rd lock
                                if (0 == strncmp(cmdRdLock, string_to_parse, sizeof(cmdRdLock)))                       // Read lock
                                {
                                    printf("Locked: %d\n\n", CHK(PINC, pllLOCK));
                                }
                                else

                                    // rd ver
                                    if (0 == strncmp(cmdRdVer, string_to_parse, sizeof(cmdRdVer)))                     // Show version number
                                    {
                                        printf("NEXT GENERATION BEACONS - VCO-PLL S/W V. %s\n\n", version);
                                    }
                                    else

                                        // wr defaults
                                        if (0 == strncmp(cmdWrDefaults, string_to_parse, sizeof(cmdWrDefaults)))       // wr defaults command
                                        {
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_FuncLatch, 0x1F);                  // PLL function latch, 00011111 10000000 10010010
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_FuncLatch + 1, 0x80);
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_FuncLatch + 2, 0x92);

                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_NCounter, 0x02);                   // PLL N counter, 00000010 01110001 00000001
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_NCounter + 1, 0x71);
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_NCounter + 2, 0x01);

                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_RCounter, 0);                      // PLL R counter, 00000000 00000000 11001000
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_RCounter + 1, 0);
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_RCounter + 2, 0xC8);

                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_BootDelay, 0);                     // Boot-up delay, 0 ms
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_SPIPulseDelay, 1);                 // SPI pulse delay, 1
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_VCOSetupDelay, 1);                 // VCO setup delay, 1
                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_MainLoopDelay, 255);               // Main loop delay, 255

                                            eeprom_write_byte((uint8_t*) EEPROM_Adr_PromptLockStatus, 0);              // Prompt lock status

                                            printf("OK\n");
                                        }
                                        else

                                            // load
                                            if (0 == strncmp(cmdLoad, string_to_parse, sizeof(cmdLoad)))               // load command
                                            {
                                                printf("OK\n");
                                                LoadSettings();
                                                SetupPLL();
                                            }
                                            else

                                                if (strlen(string_to_parse) > 0)                                       // So if not an empty string then print unknown command
                                                    printf("UNKNOW COMMAND\n");;

    printf("VCO-PLL>\n");
}

// ----------------- EOF -------------------------------------------------------------------