/*
The Next Generation Beacons software main file - VCOPLL.c

Copyright
  The OZ7IGY Beacon Next Generation Beacon platform implementation
    VCOPLL.c/h
    Poul-Erik Hansen, OZ1CKG and Bo Hansen, OZ2M

This file is part of Next Generation Beacons - VCOPLL v. 2

VCOPLL is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

VCOPLL is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details:
            www.gnu.org/licenses
*/

/*
Next Generation Beacons VCO-PLL software
----------------------------------------
Compiler: AVR Studio 6.2.1153, Windows 7

MCU: Atmel ATmega88, 8 MHz

Configuration
- Analog Devices ADF4107 Port A 7 to 1
    PB0-2 unused
    PB3 ISP MOSI, output
    PB4 ISP MISO, input
    PB5 ISP SCK, output

    PC0 PLL MuxOut, input
    PC1 PLL LE, output
    PC2 PLL Data, output/[input]
    PC3 PLL CLK, output
    PC4 PLL CE, output
    PC5 unused
    PC6 ISP reset

- FTDI FT232R USB <-> 3,3 V
    PD0(RDX0), input
    PD1(TXD0), output
    PD2-7 unused

Homepage: www.rudius.net/oz2m/ngnb

*/
#define F_CPU 8000000UL

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <util/delay.h>
#include <ctype.h>
#include <avr/pgmspace.h>

#include "VCOPLL.h"                             // Own include files
#include "Management.c"
#include "Management.h"


// ------------------- Serial and printf re-configuration -----------------------------------------------------------------
/*
  Functionality
    Reconfigures the printf function to send its output to the serial management port
    This way the all the functionality of the printf function can be used in a smooth way
*/
int SendSerialChar(char data, FILE *Unused)                                              // See example in section ? in ATmega88A datasheet
{
    while (!(UCSR0A & (1 << UDRE0)));                                                      // Wait for empty transmit buffer
    UDR0 = data;                                                                         // Put data into buffer, sends the data
    return 0;
}

void Reconfig_printf()                                                                   // Reconfigure printf to use sendSerialChar that uses USART0
{
    static FILE uart_str = FDEV_SETUP_STREAM(SendSerialChar, NULL, _FDEV_SETUP_RW);
    stdout = stdin  = &uart_str;
}

ISR(USART_RX_vect)
{
    if (CHK(UCSR0A, RXC0))                                                               // Check if character received
    {
        mgmtChar = UDR0;                                                                   // Character received so read it

        if (mgmtChar != '\n')                                                              // Collect incoming chars until LF
        {
            mgmtFrame[mgmtCharCount] = mgmtChar;
            mgmtCharCount++;
            mgmtFrame[mgmtCharCount] = 0;                                                    // Set EOS sz
        }
        else
        {
            CommandPaser(mgmtFrame);                                                         // Send input to command parser

            mgmtCharCount = 0;                                                               // Clear buffer counter
            mgmtFrame[0] = 0;
        }
    }

    if (mgmtCharCount > sizeof(mgmtFrame) - 1)                                           // Handle buffer overflow
    {
        mgmtCharCount = 0;
        mgmtFrame[0] = 0;
    }
}

// ------------------- Variable delay functions --------------------------------------------------------------------------
// Justification is that _delay_ms does not accept variables as input. i.e. input MUST be a constant

void DelayVar_ms(int16_t count)
{
    while (count-- > 0)
        _delay_ms(1);
}

void DelayVar_us(int16_t count)
{
    while (count-- > 0)
        _delay_us(1);
}

// ------------------- H/W routines -----------------------------------------------------------------------------------------

void LoadSettings()
{
    int8_t i;

    for (i = 0; i < 3; i++)
    {
        functionLatch[i] = eeprom_read_byte((uint8_t*) EEPROM_Adr_FuncLatch + i);
        nCounter[i] = eeprom_read_byte((uint8_t*) EEPROM_Adr_NCounter + i);
        rCounter[i] = eeprom_read_byte((uint8_t*) EEPROM_Adr_RCounter + i);
    }

    bootDelay = eeprom_read_byte((uint8_t*) EEPROM_Adr_BootDelay);
    spiPulseDelay = eeprom_read_byte((uint8_t*) EEPROM_Adr_SPIPulseDelay);
    vcoSetupDelay = eeprom_read_byte((uint8_t*) EEPROM_Adr_VCOSetupDelay);
    mainLoopDelay = eeprom_read_byte((uint8_t*) EEPROM_Adr_MainLoopDelay);

    promptLockStatus = eeprom_read_byte((uint8_t*) EEPROM_Adr_PromptLockStatus);
}

void SPIToPLL(uint8_t dataByte)  // this one takes MSB first
{
    int8_t bitNo = 7;

    while (bitNo >= 0)
    {
        if CHK(dataByte, bitNo)
            SET(pllPort, pllDATA);
        else
            CLR(pllPort, pllDATA);

        DelayVar_us(spiPulseDelay);  // data settle delay

        SET(pllPort, pllCLK);
        DelayVar_us(spiPulseDelay);  // clock high time
        CLR(pllPort, pllCLK);
        DelayVar_us(spiPulseDelay);  // clock low time
        bitNo--;
    }
}

void SetupPLL()
{
    int8_t i;

    CLR(pllPort, pllCE);                  // Setup PLL - CE Pin Method ADF4107 datasheet p. 17

    // Function latch
    for (i = 0; i < 3; i++)
        SPIToPLL(functionLatch[i]);
    SET(pllPort, pllLE);
    DelayVar_us(spiPulseDelay);
    CLR(pllPort, pllLE);
    DelayVar_us(spiPulseDelay);

    // N counter
    for (i = 0; i < 3; i++)
        SPIToPLL(nCounter[i]);
    SET(pllPort, pllLE);
    DelayVar_us(spiPulseDelay);
    CLR(pllPort, pllLE);
    DelayVar_us(spiPulseDelay);

    // R counter
    for (i = 0; i < 3; i++)
        SPIToPLL(rCounter[i]);
    SET(pllPort, pllLE);
    DelayVar_us(spiPulseDelay);
    CLR(pllPort, pllLE);
    DelayVar_us(spiPulseDelay);

    DelayVar_ms(5 * vcoSetupDelay);

    SET(pllPort, pllCE);
}

/*
  Functionality
    Setup the hardware environment in the MCU.

    When executed a welcome string is sent to the management port showing S/W version number

  Input
    (The hardware design)

  Output
   (Enabled ports, interrupt etc.)
*/
void HardwareInit()
{
    // Ports  76543210
    DDRB  = 0b00101000;                                                                    // Config port B
    PORTB = 0b00010111;                                                                    // Enable pull-ups for inputs and set outputs

    DDRC  = 0b00011110;                                                                    // Config port C
//  PORTC = 0b11111111;                                                                    // Enable pull-ups for inputs and set outputs

    DDRD  = 0b00000010;                                                                    // Config port D
    PORTD = 0b11111101;                                                                    // Enable pull-ups for inputs and set outputs

    // Prepare PLL
    CLR(pllPort, pllCE);
    CLR(pllPort, pllLE);
    _delay_us(10);
    SET(pllPort, pllCE);
    _delay_us(10);
    CLR(pllPort, pllCE);

    // Management UART0, 9600 baud, 8N1
    UCSR0B = 0b00000000;                                                                   // Section 19.10.3, disable interrupts and receiving while setting up
    UCSR0A = 0b00000000;                                                                   // Section 19.10.2, U2X=0 i.e. disabled
    UCSR0C = 0b00000110;                                                                   // Section 19.10.4, 0, async UMSELn=0, parity N UPM=00, stop 1 USBS=0, bits 8 UCSZ=11, rising edge UCPOL=0
    UBRR0L = 51;                                                                           // Table 19-11, 9600 baud at 8 MHz clock, UX2=0
    UBRR0H = 0;                                                                            // High part of baud rate

    UCSR0B = 0b10011000;                                                                   // Serial management, section 19.10.3, enable RX Interrupt bit 7=1, RX enable bit4 RXEN0=1, TX enable TXEN0=1, UCSZ12=0
    _delay_ms(10);                                                                         // Small delay to allow port to settle otherwise garbage data sent to serial port
    printf("\n\nNEXT GENERATION BEACONS - VCO-PLL S/W V. %s\n", version);

    sei();                                                                                 // Enable global interrupts
}

// ------------------- Main program -----------------------------------------------------------------------------------------

int main(void)
{
    _delay_ms(1000);                                                                       // Fixed delay to allow PSU to settle

    Reconfig_printf();                                                                     // printf is redirected to send output to USART0, i.e. management port

    LoadSettings();                                                                        // Load settings from the EEPROM
    DelayVar_ms(bootDelay);                                                                // Additional user configurable delay if needed
    HardwareInit();                                                                        // Initialize the H/W

    SetupPLL();

    while (1)
    {
        DelayVar_ms(5 * mainLoopDelay);

        switch (promptLockStatus)
        {
            case (0):
                break;                                                                   // Do nothing
            case (1):
                if (CHK(PINC, pllLOCK) == 0)                                             // Prompt if unlocked
                    printf("Locked: 0\n");
                break;
            case (2):
                printf("Locked: %d\n", CHK(PINC, pllLOCK));                              // Prompt unconditionally
                break;
            default:
                ;
        }
    }
}